.bk-input.filepond--root {
  background-color: var(--panel-surface-color, #f1f1f1);
  border: unset;
}
.bk-input.filepond--root .filepond--drop-label {
  color: var(--panel-on-surface-color, #000);
  background-color: var(--panel-surface-color, #f1f1f1);
}
